pub fn builtin_pattern_set(name: &str) -> Option<&'static PatternSet>Expand description
The built-in pattern set of that name, compiled on first use, or None
if there is none. builtin_pattern_set_names lists and describes them.
pub fn builtin_pattern_set(name: &str) -> Option<&'static PatternSet>The built-in pattern set of that name, compiled on first use, or None
if there is none. builtin_pattern_set_names lists and describes them.