pub fn builtin_pattern_set_names() -> &'static [&'static str]Expand description
The names of the built-in pattern sets:
"arabic-naskh": Arabic kashida insertion rules suitable for classical naskh and naskh-like typefaces (e.g. thuluth) that follow the classical rules of Arabic calligraphy and have advanced relations between letters. The rules are derived mainly from the publications of Benatia et al. on the subject, but overridden where they disagree with Fawzi Salim Afifi’s Taallum al-khatt al-arabi, part 3 (تعلم الخط العربي، الجزء الثالث)."arabic-nastaliq": Arabic kashida insertion rules suitable for classical nastaliq typefaces. Uses thearabic-naskhrules as a base with some tailoring for nastaliq. Currently based on Afifi’s Silsilat taalim al-khatt al-arabi, part 9: al-khatt al-farisi (سلسلة تعليم الخط العربي، الجزء التاسع: الخط الفارسي)."arabic-simple": Arabic kashida insertion rules suitable for “simple” typefaces, i.e. those where letters have only the basic forms with no or very limited relations between letters (no or very few ligatures, contextual alternates, and so on). It is also suitable for kufic styles of Arabic in general. The rules are based on Microsoft justification rules, which are rooted in Arabic newspaper typesetting."syriac": Syriac kashida insertion rules, following the guidelines proposed for justified Syriac.
See the .pat sources for the rules themselves and their citations.